@@ -1,30 +1,24 @@
 # -*- coding: utf-8 -*-
 
 from django.conf import settings
-if 'localeurl' in settings.INSTALLED_APPS:
-    from localeurl.models import patch_reverse
-    patch_reverse()
 
-import logging
 from datetime import datetime
-from bs4 import BeautifulSoup
+import logging
 
 from django.contrib.auth.models import User, Permission
 from django.contrib.contenttypes.models import ContentType
-from django.core.urlresolvers import reverse
 from django.test import TestCase
+from django.urls import reverse
+from django.test.utils import override_settings
 
 from model_mommy import mommy
 
-from coop_cms.apps.test_app.models import TestClass
+from coop_cms.apps.test_app.models import TestClass, TestTag
 from coop_cms import settings as coop_settings
-from coop_cms.models import BaseArticle
-
-try:
-    AUTH_LOGIN_NAME = "auth_login"
-    reverse(AUTH_LOGIN_NAME)
-except:
-    AUTH_LOGIN_NAME = "login"
+from coop_cms.models import BaseArticle, Newsletter
+from coop_cms.tests import BeautifulSoup, BaseArticleTest
+from coop_cms.tests.test_newsletter import NewsletterSettingsTest
+from coop_cms.utils import get_login_url
 
 
 class BaseTestCase(TestCase):
@@ -42,19 +36,18 @@ class BaseTestCase(TestCase):
     def _log_as_editor(self):
         self.editor = user = User.objects.create_user('editor', 'toto@toto.fr', 'editor')
         
-        ct = ContentType.objects.get_for_model(TestClass)
+        content_type = ContentType.objects.get_for_model(TestClass)
         
-        perm = 'change_{0}'.format(ct.model)
-        can_edit = Permission.objects.get(content_type=ct, codename=perm)
+        perm = 'change_{0}'.format(content_type.model)
+        can_edit = Permission.objects.get(content_type=content_type, codename=perm)
         user.user_permissions.add(can_edit)
         
-        perm = 'add_{0}'.format(ct.model)
-        can_add = Permission.objects.get(content_type=ct, codename=perm)
+        perm = 'add_{0}'.format(content_type.model)
+        can_add = Permission.objects.get(content_type=content_type, codename=perm)
         user.user_permissions.add(can_add)
         
         user.is_active = True
-        user.is_staff = True #can_edit_object
-        #user.user_permissions
+        user.is_staff = True  # can_edit_object
         user.save()
         return self.client.login(username='editor', password='editor')
   
@@ -72,12 +65,11 @@ class GenericViewTestCase(BaseTestCase):
         response = self.client.get(url)
         if coop_settings.is_perm_middleware_installed():
             self.assertEqual(302, response.status_code)
-            auth_url = reverse(AUTH_LOGIN_NAME)
-            self.assertRedirects(response, auth_url+'?next='+url)
+            auth_url = get_login_url()
+            self.assertRedirects(response, auth_url + '?next=' + url)
         else:
             self.assertEqual(403, response.status_code)
-        
-    
+
     def test_edit_object_anonymous(self):
         obj = mommy.make(TestClass)
         url = obj.get_edit_url()
@@ -85,8 +77,8 @@ class GenericViewTestCase(BaseTestCase):
         response = self.client.get(url)
         if coop_settings.is_perm_middleware_installed():
             self.assertEqual(302, response.status_code)
-            auth_url = reverse(AUTH_LOGIN_NAME)
-            self.assertRedirects(response, auth_url+'?next='+url)
+            auth_url = get_login_url()
+            self.assertRedirects(response, auth_url + '?next=' + url)
         else:
             self.assertEqual(403, response.status_code)
         
@@ -96,16 +88,15 @@ class GenericViewTestCase(BaseTestCase):
         
         if coop_settings.is_perm_middleware_installed():
             self.assertEqual(302, response.status_code)
-            auth_url = reverse(AUTH_LOGIN_NAME)
-            self.assertRedirects(response, auth_url+'?next='+url)
+            auth_url = get_login_url()
+            self.assertRedirects(response, auth_url + '?next=' + url)
         else:
             self.assertEqual(403, response.status_code)
         
         obj = TestClass.objects.get(id=obj.id)
         self.assertEqual(obj.field1, field1)
         self.assertEqual(obj.field2, field2)
-        
-        
+
     def test_view_object_viewer(self):
         self._log_as_viewer()
         obj = mommy.make(TestClass)
@@ -122,7 +113,22 @@ class GenericViewTestCase(BaseTestCase):
         soup = BeautifulSoup(response.content)
         self.assertEqual(1, len(soup.select(".bool_field_is_true")))
         self.assertEqual(0, len(soup.select(".bool_field_is_false")))
-        
+
+    def test_view_object_m2m_relationships(self):
+        self._log_as_viewer()
+        obj = mommy.make(TestClass)
+        tag1 = mommy.make(TestTag, name='The-Tag-1#')
+        tag2 = mommy.make(TestTag, name='The-Tag-2#')
+        tag3 = mommy.make(TestTag, name='The-Tag-3#')
+        obj.tags.add(tag1)
+        obj.tags.add(tag2)
+        obj.save()
+        response = self.client.get(obj.get_absolute_url())
+        self.assertEqual(200, response.status_code)
+        self.assertContains(response, tag1.name)
+        self.assertContains(response, tag2.name)
+        self.assertNotContains(response, tag3.name)
+
     def test_view_object_viewer_bool_false(self):
         self._log_as_viewer()
         obj = mommy.make(TestClass, bool_field=False)
@@ -170,8 +176,7 @@ class GenericViewTestCase(BaseTestCase):
         self.assertEqual(obj.bool_field, data["bool_field"])
         self.assertEqual(obj.int_field, data["int_field"])
         self.assertEqual(obj.float_field, data["float_field"])
-        
-        
+
     def test_edit_object_inactive(self):
         self._log_as_editor()
         self.editor.is_active = False
@@ -335,7 +340,6 @@ class FormsetViewTestCase(BaseTestCase):
         self.assertEqual(data['form-0-int_field'], obj.int_field)
         self.assertEqual(data['form-0-float_field'], obj.float_field)
         
-        
     def test_edit_formset_several_object(self):
         self._log_as_editor()
         
@@ -441,7 +445,7 @@ class FormsetViewTestCase(BaseTestCase):
         
         if coop_settings.is_perm_middleware_installed():
             self.assertEqual(302, response.status_code)
-            auth_url = reverse(AUTH_LOGIN_NAME)
+            auth_url = get_login_url()
             self.assertRedirects(response, auth_url+'?next='+url)
         else:
             self.assertEqual(403, response.status_code)
@@ -546,7 +550,7 @@ class ArticleFormTest(BaseTestCase):
                 self._settings_backup[s] = v
         
         self.LOGIN_URL = settings.LOGIN_URL
-        settings.LOGIN_URL = reverse(AUTH_LOGIN_NAME)
+        settings.LOGIN_URL = get_login_url()
         settings.COOP_CMS_NEW_ARTICLE_FORM = 'coop_cms.apps.test_app.forms.MyNewArticleForm'
         settings.COOP_CMS_ARTICLE_SETTINGS_FORM = 'coop_cms.apps.test_app.forms.MyArticleSettingsForm'
         settings.COOP_CMS_ARTICLE_TEMPLATES = (
@@ -557,10 +561,10 @@ class ArticleFormTest(BaseTestCase):
         super(ArticleFormTest, self).setUp()
         
     def tearDown(self):
-        for s in self._settings_fields_to_backup():
-            v = self._settings_backup.get(s, None)
-            if v != None:
-                setattr(settings, s, v)
+        for setting in self._settings_fields_to_backup():
+            value = self._settings_backup.get(setting, None)
+            if value is not None:
+                setattr(settings, setting, value)
         
         settings.LOGIN_URL = self.LOGIN_URL
         super(ArticleFormTest, self).tearDown()
@@ -577,7 +581,7 @@ class ArticleFormTest(BaseTestCase):
         url = reverse('coop_cms_new_article')
         response = self.client.get(url)
         self.assertEqual(302, response.status_code)
-        auth_url = reverse(AUTH_LOGIN_NAME)
+        auth_url = get_login_url()
         self.assertRedirects(response, auth_url+'?next='+url)
 
     def test_view_article_not_allowed(self):
@@ -595,8 +599,6 @@ class ArticleFormTest(BaseTestCase):
         data = {
             'title': 'test',
             'template': settings.COOP_CMS_ARTICLE_TEMPLATES[0][0],
-            #'category': '',
-            #'headline': '',
             'publication': BaseArticle.PUBLISHED,
             'in_newsletter': False,
             'navigation_parent': None,
@@ -606,11 +608,11 @@ class ArticleFormTest(BaseTestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(article_class.objects.count(), 1)
         article = article_class.objects.all()[0]
-        for f in data:
-            if f == "sites":
-                self.assertEqual([x.id for x in getattr(article, f).all()], data[f])
+        for field in data:
+            if field == "sites":
+                self.assertEqual([x.id for x in getattr(article, field).all()], data[field])
             else:
-                self.assertEqual(getattr(article, f), data[f])
+                self.assertEqual(getattr(article, field), data[field])
 
     def test_new_article_anoymous(self):
         article_class = coop_settings.get_article_class()
@@ -618,8 +620,6 @@ class ArticleFormTest(BaseTestCase):
         data = {
             'title': 'test',
             'template': settings.COOP_CMS_ARTICLE_TEMPLATES[0][0],
-            #'category': '',
-            #'headline': '',
             'publication': BaseArticle.PUBLISHED,
             'in_newsletter': False,
             'navigation_parent': None,
@@ -627,8 +627,8 @@ class ArticleFormTest(BaseTestCase):
         }
         response = self.client.post(url, data=data)
         self.assertEqual(response.status_code, 302)
-        login_url = reverse(AUTH_LOGIN_NAME)
-        self.assertTrue(response['Location'].find(login_url) > 0)
+        login_url = get_login_url()
+        self.assertTrue(response['Location'].find(login_url) >= 0)
         
         self.assertEqual(article_class.objects.count(), 0)
         
@@ -639,8 +639,6 @@ class ArticleFormTest(BaseTestCase):
         data = {
             'title': 'test',
             'template': settings.COOP_CMS_ARTICLE_TEMPLATES[0][0],
-            #'category': '',
-            #'headline': '',
             'publication': BaseArticle.PUBLISHED,
             'in_newsletter': False,
             'navigation_parent': None,
@@ -654,7 +652,7 @@ class ArticleFormTest(BaseTestCase):
     def test_view_article_settings(self):
         self._log_as_editor()
         article_class = coop_settings.get_article_class()
-        article = mommy.make(article_class)
+        article = mommy.make(article_class, slug="test")
         url = reverse('coop_cms_article_settings', args=[article.id])
         response = self.client.get(url)
         self.assertEqual(response.status_code, 200)
@@ -663,17 +661,17 @@ class ArticleFormTest(BaseTestCase):
     
     def test_view_article_settings_anonymous(self):
         article_class = coop_settings.get_article_class()
-        article = mommy.make(article_class)
+        article = mommy.make(article_class, slug="test")
         url = reverse('coop_cms_article_settings', args=[article.id])
         response = self.client.get(url)
         self.assertEqual(302, response.status_code)
-        auth_url = reverse(AUTH_LOGIN_NAME)
+        auth_url = get_login_url()
         self.assertRedirects(response, auth_url+'?next='+url)
         
     def test_view_article_settings_not_allowed(self):
         self._log_as_viewer()
         article_class = coop_settings.get_article_class()
-        article = mommy.make(article_class)
+        article = mommy.make(article_class, slug="test")
         url = reverse('coop_cms_article_settings', args=[article.id])
         response = self.client.get(url)
         self.assertEqual(response.status_code, 403)
@@ -682,7 +680,7 @@ class ArticleFormTest(BaseTestCase):
     def test_article_settings(self):
         self._log_as_editor()
         article_class = coop_settings.get_article_class()
-        article = mommy.make(article_class)
+        article = mommy.make(article_class, slug="test")
         url = reverse('coop_cms_article_settings', args=[article.id])
         
         now = datetime.now()
@@ -700,15 +698,15 @@ class ArticleFormTest(BaseTestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(article_class.objects.count(), 1)
         article = article_class.objects.all()[0]
-        for f in data:
-            if f == "sites":
-                self.assertEqual([x.id for x in getattr(article, f).all()], data[f])
+        for field in data:
+            if field == "sites":
+                self.assertEqual([x.id for x in getattr(article, field).all()], data[field])
             else:
-                self.assertEqual(getattr(article, f), data[f])
+                self.assertEqual(getattr(article, field), data[field])
         
-    def test_article_settings_anoymous(self):
+    def test_article_settings_anonymous(self):
         article_class = coop_settings.get_article_class()
-        article = mommy.make(article_class)
+        article = mommy.make(article_class, slug="test")
         url = reverse('coop_cms_article_settings', args=[article.id])
         
         data = {
@@ -722,8 +720,8 @@ class ArticleFormTest(BaseTestCase):
         }
         response = self.client.post(url, data=data)
         self.assertEqual(response.status_code, 302)
-        login_url = reverse(AUTH_LOGIN_NAME)
-        self.assertTrue(response['Location'].find(login_url)>0)
+        login_url = get_login_url()
+        self.assertTrue(response['Location'].find(login_url) >= 0)
         
         self.assertEqual(article_class.objects.count(), 1)
         article = article_class.objects.all()[0]
@@ -732,7 +730,7 @@ class ArticleFormTest(BaseTestCase):
     def test_article_settings_not_allowed(self):
         self._log_as_viewer()
         article_class = coop_settings.get_article_class()
-        article = mommy.make(article_class)
+        article = mommy.make(article_class, slug="test")
         url = reverse('coop_cms_article_settings', args=[article.id])
         
         data = {
@@ -750,3 +748,93 @@ class ArticleFormTest(BaseTestCase):
         self.assertEqual(article_class.objects.count(), 1)
         article = article_class.objects.all()[0]
         self.assertNotEqual(article.summary, data['summary'])
+
+
+class MyNewsletterSettingsTest(NewsletterSettingsTest):
+
+    @override_settings(COOP_CMS_NEWSLETTER_SETTINGS_FORM="coop_cms.apps.test_app.forms.MyNewsletterSettingsForm")
+    def test_additional_field_on_edit(self):
+        self._log_as_editor()
+        newsletter = mommy.make(
+            Newsletter,
+            subject="a little intro for this newsletter",
+            template="test/newsletter_blue.html",
+        )
+
+        url = reverse("coop_cms_newsletter_settings", args=[newsletter.id])
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        soup = BeautifulSoup(response.content)
+        self.assertEqual(1, len(soup.select("#id_dummy")))
+
+    @override_settings(COOP_CMS_NEWSLETTER_SETTINGS_FORM="coop_cms.apps.test_app.forms.MyNewsletterSettingsForm")
+    def test_additional_field_on_create(self):
+        self._log_as_editor()
+
+        url = reverse("coop_cms_new_newsletter")
+
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        soup = BeautifulSoup(response.content)
+        self.assertEqual(1, len(soup.select("#id_dummy")))
+
+
+@override_settings(COOP_CMS_ARTICLE_TEMPLATES=(('coop_cms/test_app/custom_tag_template.html', 'Custom Tag'),))
+class CustomTemplateTagInCmsEditTag(BaseArticleTest):
+    """test using custom templatetag inside the cms_edit template tag"""
+
+    def test_view_with_blocks(self):
+        """test view article with block templatetag inside the cms_edit template tag"""
+
+        article_class = coop_settings.get_article_class()
+        article = mommy.make(
+            article_class,
+            title="This is my article", content="<p>This is my <b>content</b></p>",
+            template='coop_cms/test_app/custom_tag_template.html'
+        )
+
+        response = self.client.get(article.get_absolute_url())
+        self.assertEqual(response.status_code, 200)
+
+        soup = BeautifulSoup(response.content)
+
+        self.assertEqual(3, len(soup.select("ul.custom li")))
+
+        self.assertContains(response, article.title)
+        self.assertContains(response, article.content)
+
+        self.assertContains(response, "*** HELLO FROM CHILD ***")
+        self.assertContains(response, "*** HELLO FROM PARENT ***")
+        self.assertContains(response, "*** HELLO FROM BLOCK ***")
+
+    def test_edit_with_blocks(self):
+        """test edition with block templatetag inside the cms_edit template tag"""
+
+        article_class = coop_settings.get_article_class()
+        article = mommy.make(
+            article_class,
+            title="This is my article", content="<p>This is my <b>content</b></p>",
+            template='coop_cms/test_app/custom_tag_template.html'
+        )
+
+        self._log_as_editor()
+
+        data = {
+            "title": "This is a new title",
+            'content': "<p>This is a <i>*** NEW ***</i> <b>content</b></p>"
+        }
+        response = self.client.post(article.get_edit_url(), data=data, follow=True)
+        self.assertEqual(response.status_code, 200)
+
+        article = article_class.objects.get(id=article.id)
+
+        self.assertEqual(article.title, data['title'])
+        self.assertEqual(article.content, data['content'])
+
+        self.assertContains(response, article.title)
+        self.assertContains(response, article.content)
+
+        self.assertContains(response, "*** HELLO FROM CHILD ***")
+        self.assertContains(response, "*** HELLO FROM PARENT ***")
+        self.assertContains(response, "*** HELLO FROM BLOCK ***")
