@@ -1,13 +1,13 @@
 # -*- coding: utf-8 -*-
 
 from django.conf import settings
-if 'localeurl' in settings.INSTALLED_APPS:
-    from localeurl.models import patch_reverse
-    patch_reverse()
 
-from django.utils.unittest import SkipTest
+from coop_cms.moves import SkipTest
 
-from coop_cms.apps.test_app.tests import GenericViewTestCase as BaseGenericViewTestCase
+if 'coop_cms.apps.test_app' in settings.INSTALLED_APPS:
+    from coop_cms.apps.test_app.tests import GenericViewTestCase as BaseGenericViewTestCase
+else:
+    from coop_cms.tests import BaseTestCase as BaseGenericViewTestCase
 
 
 class GenericViewTestCase(BaseGenericViewTestCase):
@@ -20,5 +20,4 @@ class GenericViewTestCase(BaseGenericViewTestCase):
     def setUp(self):
         super(GenericViewTestCase, self).setUp()
         if not ('coop_cms.apps.test_app' in settings.INSTALLED_APPS):
-            print self.warning
-            raise SkipTest()
+            raise SkipTest('coop_cms.apps.test_app not installed')
